@@ -9,19 +9,18 @@ import sys
 import json
 
 from clickable.utils import (
-    run_subprocess_call,
     run_subprocess_check_call,
     run_subprocess_check_output,
-    check_command,
     image_exists,
+    env,
+    check_command,
 )
 from clickable.logger import logger
-from clickable.config.project import ProjectConfig
 from clickable.config.constants import Constants
 from clickable.exceptions import ClickableException
 
 
-class Container(object):
+class Container():
     def __init__(self, config, name=None, minimum_version=None):
         self.config = config
         self.docker_mode = self.config.needs_docker()
@@ -29,22 +28,26 @@ class Container(object):
         self.docker_image = self.config.docker_image
         self.base_docker_image = self.docker_image
 
-        if self.config.needs_clickable_image():
-            self.clickable_dir = '.clickable/{}'.format(self.config.build_arch)
+        if self.docker_mode:
+            self.clickable_dir = f'.clickable/{self.config.build_arch}'
             if name:
-                self.clickable_dir = '{}/{}'.format(self.clickable_dir, name)
+                self.clickable_dir = f'{self.clickable_dir}/{name}'
 
-            self.docker_name_file = '{}/image.json'.format(self.clickable_dir)
-            self.docker_file = '{}/Dockerfile'.format(self.clickable_dir)
+            self.docker_name_file = f'{self.clickable_dir}/image.json'
+            self.docker_file = f'{self.clickable_dir}/Dockerfile'
 
             if self.needs_customized_container():
                 self.restore_cached_image()
 
+        if self.config.builder == Constants.RUST and self.config.cargo_home:
+            logger.info("Caching cargo related files in %s",
+                        self.config.cargo_home)
+
     def restore_cached_image(self):
         if not os.path.exists(self.docker_name_file):
             return
 
-        with open(self.docker_name_file, 'r') as f:
+        with open(self.docker_name_file, 'r', encoding='UTF-8') as f:
             cached_image = None
             cached_base_image = None
 
@@ -68,8 +71,8 @@ class Container(object):
 
             self.check_docker()
 
-            command_base = 'docker images -q {}'.format(self.base_docker_image)
-            command_cached = 'docker history -q {}'.format(cached_image)
+            command_base = f'docker images -q {self.base_docker_image}'
+            command_cached = f'docker history -q {cached_image}'
 
             hash_base = run_subprocess_check_output(command_base).strip()
             history_cached = run_subprocess_check_output(command_cached).strip()
@@ -81,39 +84,68 @@ class Container(object):
                 logger.warning("Cached container is outdated")
 
     def start_docker(self):
-        started = False
-        error_code = run_subprocess_call(shlex.split('which systemctl'), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        check_command('systemctl')
 
-        if error_code == 0:
-            logger.info('Asking for root to start docker')
-            error_code = run_subprocess_call(shlex.split('sudo systemctl start docker'))
+        logger.info('Asking for root to start docker')
+        run_subprocess_check_output(shlex.split('sudo systemctl start docker'))
 
-            started = (error_code == 0)
+    def is_docker_service_running(self):
+        if env('CLICKABLE_SKIP_DOCKER_CHECKS'):
+            return True
 
-        return started
+        check_command('systemctl')
+
+        try:
+            run_subprocess_check_output(
+                shlex.split('systemctl is-active --quiet snap.docker.dockerd.service'),
+                stderr=subprocess.STDOUT)
+            return True
+        except subprocess.CalledProcessError:
+            pass
+
+        try:
+            run_subprocess_check_output(
+                shlex.split('systemctl is-active --quiet docker'),
+                stderr=subprocess.STDOUT)
+            return True
+        except subprocess.CalledProcessError:
+            pass
+
+        return False
 
     def check_docker(self, retries=3):
         if not self.docker_mode:
-            raise ClickableException("Container was not initialized with Container Mode. This seems to be a bug in Clickable.")
+            raise ClickableException(
+                "Container was not initialized with Container Mode. "
+                "This seems to be a bug in Clickable."
+            )
 
-        if self.needs_docker_setup():
-            self.setup_docker()
+        check_command('docker')
 
-        try:
-            run_subprocess_check_output(shlex.split('docker ps'), stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
+        if not (self.needs_docker_setup() and self.is_systemd_used()):
+            return
+
+        self.setup_docker()
+
+        if not self.is_docker_service_running():
             retries -= 1
             if retries <= 0:
-                raise ClickableException("Couldn't check docker. If you just installed Clickable you may need to reboot once.")
+                raise ClickableException(
+                    "Couldn't check docker. If you just installed Clickable you may "
+                    "need to reboot once."
+                )
 
             self.start_docker()
 
             time.sleep(3)  # Give it a sec to boot up
             self.check_docker(retries)
 
+    def is_systemd_used(self):
+        return subprocess.call('command -v systemctl >> /dev/null', shell=True) == 0
+
     def docker_group_exists(self):
         group_exists = False
-        with open('/etc/group', 'r') as f:
+        with open('/etc/group', 'r', encoding='UTF-8') as f:
             lines = f.readlines()
             for line in lines:
                 if line.startswith('docker:'):
@@ -121,34 +153,56 @@ class Container(object):
 
         return group_exists
 
-    def user_part_of_docker_group(self):
-        output = run_subprocess_check_output(shlex.split('groups {}'.format(getpass.getuser()))).strip()
+    def user_in_docker_group_pending(self):
+        return self.user_in_docker_group() and not self.proccess_in_docker_group()
 
-        # Test for exactly docker in the group list
-        return (' docker ' in output or output.endswith(' docker') or output.startswith('docker ') or output == 'docker')
+    def proccess_in_docker_group(self):
+        check_command('groups')
+
+        groups = run_subprocess_check_output(shlex.split('groups')).strip().split()
+
+        return 'docker' in groups
+
+    def user_in_docker_group(self):
+        check_command('groups')
+
+        groups = run_subprocess_check_output(
+            shlex.split(f'groups {getpass.getuser()}')
+        ).strip().split(':')[1].split()
+
+        return 'docker' in groups
 
     def needs_docker_setup(self):
         return (
-            sys.platform != 'darwin' and
-            (not self.docker_group_exists() or not self.user_part_of_docker_group())
+            not env('CLICKABLE_SKIP_DOCKER_CHECKS') and
+            not self.is_docker_ready()
         )
 
     def setup_docker(self):
         logger.info('Setting up docker')
 
-        self.start_docker()
+        if not self.is_docker_service_running():
+            self.start_docker()
+
+        if self.is_docker_ready():
+            logger.info('Setup has already been completed')
+            return
 
         if not self.docker_group_exists():
             logger.info('Asking for root to create docker group')
             subprocess.check_call(shlex.split('sudo groupadd docker'))
 
-        if self.user_part_of_docker_group():
-            logger.info('Setup has already been completed')
-        else:
+        if not self.user_in_docker_group():
             logger.info('Asking for root to add the current user to the docker group')
-            subprocess.check_call(shlex.split('sudo usermod -aG docker {}'.format(getpass.getuser())))
+            subprocess.check_call(
+                shlex.split(f'sudo usermod -aG docker {getpass.getuser()}')
+            )
 
-            raise ClickableException('Log out or restart to apply changes')
+        if self.user_in_docker_group_pending():
+            raise ClickableException('Log out or restart to gain docker access')
+
+    def is_docker_ready(self):
+        return self.docker_group_exists() and self.proccess_in_docker_group()
 
     def pull_files(self, files, dst_parent):
         os.makedirs(dst_parent, exist_ok=True)
@@ -167,27 +221,26 @@ class Container(object):
         else:  # Docker
             mounts = self.render_mounts(
                 self.get_docker_mounts(transparent=[self.config.root_dir]))
-            command_create = 'docker create {} {}'.format(
-                mounts, self.docker_image)
+            command_create = f'docker create {mounts} {self.docker_image}'
             container = run_subprocess_check_output(command_create).strip()
 
             for f in files:
-                command_copy = 'docker cp {}:{} {}'.format(
-                    container, f, dst_parent)
+                command_copy = f'docker cp {container}:{f} {dst_parent}'
                 run_subprocess_check_call(command_copy)
 
-            command_remove = 'docker rm {}'.format(container)
+            command_remove = f'docker rm {container}'
             run_subprocess_check_call(command_remove,
                                       stdout=subprocess.DEVNULL)
 
-    def get_docker_mounts(self, transparent=[]):
+    def get_docker_mounts(self, transparent=None):
         # container path is key, host path is value
         mounts = {}
 
         if self.config.builder == Constants.GO and self.config.gopath:
             gopaths = self.config.gopath.split(':')
             for (index, path) in enumerate(gopaths):
-                mounts['/gopath/path{}'.format(index)] = path
+                mounts[f'/gopath/path{index}'] = path
+                os.makedirs(path, exist_ok=True)
 
         if self.config.builder == Constants.RUST and self.config.cargo_home:
             cargo_registry = os.path.join(self.config.cargo_home, 'registry')
@@ -199,21 +252,22 @@ class Container(object):
             os.makedirs(cargo_git, exist_ok=True)
 
             # create .package-cache if it doesn't exist
-            with open(cargo_package_cache_lock, "a"):
+            with open(cargo_package_cache_lock, "a", encoding='UTF-8'):
                 pass
 
             mounts['/opt/rust/cargo/registry'] = cargo_registry
             mounts['/opt/rust/cargo/git'] = cargo_git
             mounts['/opt/rust/cargo/.package-cache'] = cargo_package_cache_lock
 
-        for path in transparent:
-            mounts[path] = path
+        if transparent:
+            for path in transparent:
+                mounts[path] = path
 
         return mounts
 
     def render_mounts(self, mounts):
         return " ".join([
-            "-v {}:{}:Z".format(host, container)
+            f"-v {host}:{container}:Z"
             for container, host in mounts.items()
         ])
 
@@ -229,53 +283,40 @@ class Container(object):
         cwd = cwd if cwd else os.path.abspath(self.config.root_dir)
 
         if self.config.container_mode:
-            wrapped_command = 'bash -c "{}"'.format(command)
+            wrapped_command = f'bash -c "{command}"'
         else:  # Docker
             self.check_docker()
 
-            if ' ' in cwd or ' ' in self.config.build_dir:
-                raise ClickableException('There are spaces in the current path, this will cause errors in the build process')
-
             if self.config.first_docker_info:
-                logger.debug('Using docker container "{}"'.format(self.docker_image))
+                logger.debug('Using docker container "%s"', self.docker_image)
                 self.config.first_docker_info = False
 
             go_config = ''
             if self.config.builder == Constants.GO and self.config.gopath:
                 gopaths = self.config.gopath.split(':')
                 docker_gopaths = [
-                    '/gopath/path{}'.format(index)
+                    f'/gopath/path{index}'
                     for index in range(len(gopaths))
                 ]
-                go_config = '-e GOPATH={}'.format(':'.join(docker_gopaths), )
-
-            rust_config = ''
-
-            if self.config.builder == Constants.RUST and self.config.cargo_home:
-                logger.info("Caching cargo related files in {}".format(
-                    self.config.cargo_home))
+                joined_go_paths = ':'.join(docker_gopaths)
+                go_config = f'-e GOPATH={joined_go_paths}'
 
             env_vars = self.config.prepare_docker_env_vars()
 
             user = ""
             if not root_user:
-                user = "-u {}".format(os.getuid())
+                user = f"-u {os.getuid()}"
 
             mounts = self.render_mounts(
-                self.get_docker_mounts(transparent=[cwd]))
+                self.get_docker_mounts(transparent=[cwd, self.config.root_dir]))
 
-            wrapped_command = 'docker run {mounts} {env} {go} {rust} {user} -w {cwd} --rm {tty} {network} -i {image} bash -c "{cmd}"'.format(
-                mounts=mounts,
-                env=env_vars,
-                go=go_config,
-                rust=rust_config,
-                cwd=self.config.build_dir if use_build_dir else cwd,
-                user=user,
-                image=self.docker_image,
-                cmd=command,
-                tty="-t" if tty else "",
-                network='--network="host"' if localhost else "",
-            )
+            command_cwd = self.config.build_dir if use_build_dir else cwd
+            command_tty = "-t" if tty else ""
+            network = '--network="host"' if localhost else ""
+
+            wrapped_command = f'docker run {mounts} {env_vars} {go_config} {user} ' \
+                              f'-w {command_cwd} --rm {command_tty} {network} ' \
+                              f'-i {self.docker_image} bash -c "{command}"'
 
         kwargs = {}
         if use_build_dir:
@@ -283,8 +324,9 @@ class Container(object):
 
         if get_output:
             return run_subprocess_check_output(shlex.split(wrapped_command), **kwargs)
-        else:
-            subprocess.check_call(shlex.split(wrapped_command), **kwargs)
+
+        subprocess.check_call(shlex.split(wrapped_command), **kwargs)
+        return None
 
     def get_dependency_packages(self):
         dependencies = self.config.dependencies_host
@@ -292,13 +334,13 @@ class Container(object):
             if ':' in dep:
                 dependencies.append(dep)
             else:
-                dependencies.append('{}:{}'.format(dep, self.config.arch))
+                dependencies.append(f'{dep}:{self.config.arch}')
         return dependencies
 
     def get_ppa_adding_commands(self):
         if self.config.dependencies_ppa:
             return [
-                'add-apt-repository -y {}'.format(ppa)
+                f'add-apt-repository -y {ppa}'
                 for ppa in self.config.dependencies_ppa
             ]
 
@@ -306,40 +348,42 @@ class Container(object):
 
     def construct_dockerfile_content(self, commands, env_vars):
         env_strings = [
-            'ENV {}="{}"'.format(key, var) for key,var in env_vars.items()
+            f'ENV {key}="{var}"' for key, var in env_vars.items()
         ]
 
         run_strings = [
-            'RUN {}'.format(cmd) for cmd in commands
+            f'RUN {cmd}' for cmd in commands
         ]
 
-        return '''
-FROM {}
-{}
-{}
-        '''.format(
-            self.base_docker_image,
-            '\n'.join(env_strings),
-            '\n'.join(run_strings)
-        ).strip()
+        env_lines = '\n'.join(env_strings)
+        run_lines = '\n'.join(run_strings)
+
+        return f'''
+FROM {self.base_docker_image}
+{env_lines}
+{run_lines}
+        '''.strip()
 
     def create_custom_container(self, dockerfile_content):
         if not os.path.exists(self.clickable_dir):
             os.makedirs(self.clickable_dir)
 
-        with open(self.docker_file, 'w') as f:
+        with open(self.docker_file, 'w', encoding='UTF-8') as f:
             f.write(dockerfile_content)
 
-        self.docker_image = '{}-{}'.format(self.base_docker_image, uuid.uuid4())
-        with open(self.docker_name_file, 'w') as f:
+        self.docker_image = f'{self.base_docker_image}-{uuid.uuid4()}'
+        with open(self.docker_name_file, 'w', encoding='UTF-8') as f:
             json.dump({
                 'name': self.docker_image,
                 'base_image': self.base_docker_image,
-            },f)
+            }, f)
 
         logger.debug('Generating new docker image')
         try:
-            subprocess.check_call(shlex.split('docker build -t {} .'.format(self.docker_image)), cwd=self.clickable_dir)
+            subprocess.check_call(
+                shlex.split(f'docker build -t {self.docker_image} .'),
+                cwd=self.clickable_dir
+            )
         except subprocess.CalledProcessError:
             self.clean_clickable()
             raise
@@ -354,20 +398,20 @@ FROM {}
         if not os.path.exists(self.docker_file):
             return True
 
-        with open(self.docker_file, 'r') as f:
+        with open(self.docker_file, 'r', encoding='UTF-8') as f:
             if dockerfile_content.strip() != f.read().strip():
                 return True
 
-        command = 'docker images -q {}'.format(self.docker_image)
-        image_exists = run_subprocess_check_output(command).strip()
-        return not image_exists
+        command = f'docker images -q {self.docker_image}'
+        exists = run_subprocess_check_output(command).strip()
+        return not exists
 
     def get_apt_install_cmd(self, dependencies):
-        return 'apt-get install -y --force-yes --no-install-recommends {}'.format(
-                ' '.join(dependencies))
+        joined_deps = ' '.join(dependencies)
+        return f'apt-get install -y --force-yes --no-install-recommends {joined_deps}'
 
     def setup_customized_image(self):
-        logger.debug('Checking dependencies and container setup')
+        logger.debug('Checking dependencies and image setup')
 
         self.check_docker()
 
@@ -379,10 +423,18 @@ FROM {}
         dependencies = self.get_dependency_packages()
         if dependencies:
             commands.append(
-                'echo set debconf/frontend Noninteractive | debconf-communicate && echo set debconf/priority critical | debconf-communicate')
+                'echo set debconf/frontend Noninteractive | debconf-communicate && '
+                'echo set debconf/priority critical | debconf-communicate'
+            )
+            dependencies_cmd = self.get_apt_install_cmd(dependencies)
             commands.append(
-                'apt-get update && {} && apt-get clean'.format(
-                    self.get_apt_install_cmd(dependencies)))
+                f'apt-get update && {dependencies_cmd} && apt-get clean')
+
+        if self.config.rust_channel:
+            commands.append(f'rustup default {self.config.rust_channel}')
+
+            if self.config.is_foreign_target():
+                commands.append(f'rustup target add {self.config.arch_rust}')
 
         if self.config.image_setup:
             commands.extend(self.config.image_setup.get('run', []))
@@ -407,7 +459,11 @@ FROM {}
             for dep in dependencies:
                 exists = ''
                 try:
-                    exists = self.run_command('dpkg -s {} | grep Status'.format(dep), get_output=True, use_build_dir=False)
+                    exists = self.run_command(
+                        f'dpkg -s {dep} | grep Status',
+                        get_output=True,
+                        use_build_dir=False
+                    )
                 except subprocess.CalledProcessError:
                     exists = ''
 
@@ -416,20 +472,30 @@ FROM {}
                     break
 
             if run:
-                self.run_command(self.get_apt_install_cmd(dependencies),
-                        use_build_dir=False)
+                self.run_command(
+                    self.get_apt_install_cmd(dependencies),
+                    use_build_dir=False
+                )
             else:
                 logger.debug('Dependencies already installed')
 
+        if self.config.rust_channel:
+            self.run_command(f'rustup default {self.config.rust_channel}')
+
+            if self.config.is_foreign_target():
+                self.run_command(f'rustup target add {self.config.arch_rust}')
+
         if self.config.image_setup:
             for command in self.config.image_setup.get('run', []):
                 self.run_command(command, use_build_dir=False)
 
     def needs_customized_container(self):
-        return self.config.dependencies_host \
-            or self.config.dependencies_target \
-            or self.config.dependencies_ppa \
+        return not self.config.skip_image_setup and (
+            self.config.dependencies_host
+            or self.config.dependencies_target
+            or self.config.dependencies_ppa
             or self.config.image_setup
+            or self.config.rust_channel)
 
     def check_base_image_version(self):
         if not self.minimum_version:
@@ -441,27 +507,29 @@ FROM {}
         version = 0
         try:
             format_string = '{{ index .Config.Labels "image_version"}}'
-            command = "docker inspect --format '{}' {}".format(format_string,
-                    self.docker_image)
-            logger.debug('Checking docker image version via: {}'.format(command))
+            command = f"docker inspect --format '{format_string}' {self.docker_image}"
+            logger.debug('Checking docker image version via: %s', command)
 
             version_string = run_subprocess_check_output(command)
             version = int(version_string)
         except (ValueError, subprocess.CalledProcessError):
-            logger.warn("Could not read the image version from the container")
+            logger.warning("Could not read the image version from the container")
 
         if version < self.minimum_version:
-            raise ClickableException('This version of Clickable requires Clickable docker image {} in version {} or higher (found version {}). Please run "clickable update" to update your local images.'.format(self.docker_image, self.minimum_version, version))
-
+            raise ClickableException(
+                f'This version of Clickable requires Clickable docker image {self.docker_image} '
+                f'in version {self.minimum_version} or higher (found version {version}). '
+                'Please run "clickable update-images" to update your local images.'
+            )
 
     def setup(self):
-        if self.config.container_mode:
-            self.setup_container_mode()
-
-        if self.config.needs_clickable_image():
+        if self.docker_mode:
             self.check_base_image_version()
 
-            if self.needs_customized_container():
+        if not self.config.skip_image_setup:
+            if self.config.container_mode:
+                self.setup_container_mode()
+            elif self.needs_customized_container():
                 self.setup_customized_image()
 
     def clean_clickable(self):
@@ -469,9 +537,10 @@ FROM {}
         if os.path.exists(path):
             try:
                 shutil.rmtree(path)
-            except Exception:
-                type, value, traceback = sys.exc_info()
-                if type == OSError and 'No such file or directory' in value:  # TODO see if there is a proper way to do this
+            except Exception:  # pylint: disable=broad-except
+                typ, value, _ = sys.exc_info()
+                # TODO see if there is a proper way to do this
+                if typ == OSError and 'No such file or directory' in value:
                     pass  # Nothing to do here, the directory didn't exist
                 else:
                     raise
